@@ -12,6 +12,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <arpa/inet.h>
+
 /* Configuration data
  */
 conf_data conf = CONF_INIT;
@@ -78,6 +80,37 @@ conf_device_list_lookup (const char *name) {
     return dev;
 }
 
+/* Revert conf.blacklist list
+ */
+static void
+conf_blacklist_revert (void)
+{
+    conf_blacklist *list = conf.blacklist, *prev = NULL, *next;
+
+    while (list != NULL) {
+        next = list->next;
+        list->next = prev;
+        prev = list;
+        list = next;
+    }
+
+    conf.blacklist = prev;
+}
+/* Free conf.blacklist
+ */
+static void
+conf_blacklist_free (void)
+{
+    while (conf.blacklist != NULL) {
+        conf_blacklist *next = conf.blacklist->next;
+
+        mem_free((char*) conf.blacklist->name);
+        mem_free((char*) conf.blacklist->model);
+
+        conf.blacklist = next;
+    }
+}
+
 /* Expand path name. The returned string must be eventually
  * released with mem_free()
  */
@@ -162,6 +195,66 @@ conf_load_bool (const inifile_record *rec, bool *out,
     }
 }
 
+/* Parse network address with mask
+ */
+static void
+conf_load_netaddr (const inifile_record *rec, ip_network *net)
+{
+    char *addr, *mask;
+    int  af;
+    int  maxmask;
+
+    memset(net, 0, sizeof(*net));
+
+    /* Split into address and mask */
+    addr = alloca(strlen(rec->value) + 1);
+    strcpy(addr, rec->value);
+
+    mask = strchr(addr, '/');
+    if (mask != NULL) {
+        *mask = '\0';
+        mask ++;
+    }
+
+    /* Parse address */
+    if (strchr(addr, ':') == NULL) {
+        af = AF_INET;
+        maxmask = 32;
+    } else {
+        af = AF_INET6;
+        maxmask = 128;
+    }
+
+    if (inet_pton(af, addr, &net->addr.ip) != 1) {
+        conf_perror(rec, "invalid IP address %s", addr);
+        return;
+    }
+
+    /* Parse mask, if any */
+    if (mask != NULL) {
+        unsigned long l;
+        char          *end;
+
+        l = strtoul(mask, &end, 10);
+        if (end == mask || *end != '\0') {
+            conf_perror(rec, "invalid network mask %s", mask);
+            return;
+        }
+
+        if (l == 0 || l > (unsigned long) maxmask) {
+            conf_perror(rec, "network mask out of range");
+            return;
+        }
+
+        net->mask = (int) l;
+    } else {
+        net->mask = maxmask;
+    }
+
+    /* Indicate success; all other return values already filled */
+    net->addr.af = af;
+}
+
 /* Load configuration from opened inifile
  */
 static void
@@ -213,6 +306,29 @@ conf_load_from_ini (inifile *ini)
                 } else if (inifile_match_name(rec->variable, "enable")) {
                     conf_load_bool(rec, &conf.dbg_enabled, "true", "false");
                 }
+            } else if (inifile_match_name(rec->section, "blacklist")) {
+                conf_blacklist *ent = NULL;
+
+                if (inifile_match_name(rec->variable, "name")) {
+                    ent = mem_new(conf_blacklist, 1);
+                    ent->name = str_dup(rec->value);
+                } else if (inifile_match_name(rec->variable, "model")) {
+                    ent = mem_new(conf_blacklist, 1);
+                    ent->model = str_dup(rec->value);
+                } else if (inifile_match_name(rec->variable, "ip")) {
+                    ip_network net;
+
+                    conf_load_netaddr(rec, &net);
+                    if (net.addr.af != AF_UNSPEC) {
+                        ent = mem_new(conf_blacklist, 1);
+                        ent->net = net;
+                    }
+                }
+
+                if (ent != NULL) {
+                    ent->next = conf.blacklist;
+                    conf.blacklist = ent;
+                }
             }
             break;
 
@@ -352,6 +468,7 @@ conf_load (void)
 
     /* Cleanup and exit */
     conf_device_list_revert();
+    conf_blacklist_revert();
 
     mem_free(dir_list);
     mem_free(path);
@@ -364,6 +481,7 @@ void
 conf_unload (void)
 {
     conf_device_list_free();
+    conf_blacklist_free();
     mem_free((char*) conf.dbg_trace);
     mem_free((char*) conf.socket_dir);
     conf = conf_init;
